package net.pingex.dcf.commands;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.util.*;

/**
 * Command bank
 */
public class CommandRegistry
{
	/**
	 * Command main store
	 */
	private static Set<Command> commandSet = new HashSet<>();

	/**
	 * Logger
	 */
	private static final Logger LOGGER = LogManager.getLogger(CommandRegistry.class);

	/**
	 * Register a command and its aliases to the bank
	 * @param toRegister Command to register
	 */
	public static void registerCommand(Command toRegister)
	{
		LOGGER.debug("Attempting to register command {}.", toRegister.getName());

		if(commandExists(toRegister.getName()))
		{
			LOGGER.warn("Fail to add command: {} is already registered as a command.", toRegister.getName());
			return;
		}
		if(aliasExists(toRegister.getName())) LOGGER.info("Overriding registered alias {} for a new command with the same name.", toRegister.getName());

		commandSet.add(toRegister);
		LOGGER.info("Registered command {}.", toRegister.getName());
	}

	/**
	 * Remove a command from the bank.
	 * A command cannot be removed using one of its alias.
	 * @param commandName Target command
	 */
	public static void unregisterCommandByName(String commandName)
	{
		LOGGER.debug("Resolving command {}.", commandName);
		Optional<Command> target = getCommandByName(commandName);

		if(target.isPresent()) unregisterCommand(target.get());
		else LOGGER.warn("Attempting to remove a command which is unknown to DCF.");
	}

	/**
	 * Remove a command from the bank.
	 * @param toUnregister Target command
	 */
	public static void unregisterCommand(Command toUnregister)
	{
		LOGGER.debug("Attempting to remove command {} from the bank.", toUnregister.getName());
		if(!commandSet.contains(toUnregister))
		{
			LOGGER.warn("Attempting to remove a command which is unknown to DCF.");
			return;
		}

		commandSet.remove(toUnregister);
		LOGGER.info("Removed command {}.", toUnregister.getName());
	}

	/**
	 * Get the current state of the registry.
	 * @return RO set of the bank.
	 */
	public static Set<Command> getRegistry()
	{
		return Collections.unmodifiableSet(commandSet);
	}

	/**
	 * Tells if the command is registered (excluding aliases).
	 * @param commandName Target command
 	 * @return `true` if the command is registered, `false` otherwise
	 */
	public static boolean commandExists(String commandName)
	{
		return commandSet.stream().anyMatch(e -> e.getName().equalsIgnoreCase(commandName));
	}

	/**
	 * Tells if the alias is registered.
	 * @param aliasName Target alias
	 * @return `true` if the alias is registered, `false` otherwise
	 */
	public static boolean aliasExists(String aliasName)
	{
		return commandSet.stream().anyMatch(e -> e.getAliases().stream().anyMatch(f -> f.equalsIgnoreCase(aliasName)));
	}

	/**
	 * Tells if the command exists (including aliases).
	 * Command have precedence over aliases.
	 * @param commandName Target command
	 * @return `true` if the command exists, `false` otherwise
	 */
	public static boolean exists(String commandName)
	{
		return commandExists(commandName) || aliasExists(commandName);
	}

	/**
	 * Get a command or an alias designed by its name. Commands have precedence over aliases.
	 * @param commandName Target command
	 * @return {@code Optional.empty()} if no command was found, {@code Optional.of(Command)} otherwise
	 */
	public static Optional<Command> getCommandOrAliasByName(String commandName)
	{
		return commandExists(commandName) ? getCommandByName(commandName) : getAliasByName(commandName);
	}

	/**
	 * Get a command by one of its aliases.
	 * @param aliasName Target command with this alias.
	 * @return {@code Optional.empty()} if no command was found, {@code Optional.of(Command)} otherwise
	 */
	public static Optional<Command> getAliasByName(String aliasName)
	{
		return commandSet.stream().filter(e -> e.getAliases().stream().anyMatch(f -> f.equalsIgnoreCase(aliasName))).findFirst();
	}

	/**
	 * Get a command designed by its name, excluding aliases.
	 * @param commandName Target command
	 * @return {@code Optional.empty()} if no command was found, {@code Optional.of(Command)} otherwise
	 */
	public static Optional<Command> getCommandByName(String commandName)
	{
		return commandSet.stream().filter(e -> e.getName().equalsIgnoreCase(commandName)).findFirst();
	}
}
