package net.pingex.dcf.permissions;

import net.pingex.dcf.commands.Command;
import sx.blah.discord.handle.obj.IGuild;
import sx.blah.discord.handle.obj.IMessage;
import sx.blah.discord.handle.obj.IRole;
import sx.blah.discord.handle.obj.IUser;

/**
 * Landing class of this package.
 */
public class PermissionsHandler
{
	private static final ICommandPermissionsProvider CURRENT_PROVIDER = new DefaultPermissionsProvider();

	/**
	 * Get provider used to check permissions.
	 */
	public static ICommandPermissionsProvider getProvider()
	{
		return CURRENT_PROVIDER;
	}

	/**
	 * Tells whether someone can run requested command.
	 *
	 * The validator will successively test the following until something other than `null` is returned.
	 * - Individual user permission on the guild using {@code ICommandPermissionsProvider.validateUser()}
	 * - Individual user permission globally using {@code ICommandPermissionsProvider.validateUser()}
	 * - The user's role list using {@code ICommandPermissionsProvider.validateGroup()}
	 * - Default behavior included in a Command object
	 *
	 * @param targetUser User to check against
	 * @param targetGuild Guild to check against
	 * @param command Requested command, as parsed by {@code CommandHandler}
	 * @return `true` if request is granted, `false` if denied.
	 */
	public static boolean canRunCommand(IUser targetUser, IGuild targetGuild, Command command)
	{
		// First check: user permission for guild, skipped if DM
		if(targetGuild != null)
		{
			Boolean canUserGuildRun = CURRENT_PROVIDER.validateUser(targetGuild, targetUser, command);
			if(canUserGuildRun != null) return canUserGuildRun;
		}

		// Second check: user permission globally
		Boolean canUserRun = CURRENT_PROVIDER.validateUser(null, targetUser, command);
		if(canUserRun != null) return canUserRun;

		// Third check: user role permissions, skipped if DM
		if(targetGuild != null && targetGuild.getRolesForUser(targetUser) != null)
			for(IRole i : targetGuild.getRolesForUser(targetUser))
			{
				Boolean canRoleRun = CURRENT_PROVIDER.validateGroup(i, command);
				if(canRoleRun != null) return canRoleRun;
			}

		// Fourth check: default behavior
		return command.getDefaultPermission().test(new DefaultPermission.Tuple(targetUser, targetGuild));
	}

	/**
	 * Shortcut to the other {@code canRunCommand()} method.
	 * @param request Incoming message
	 * @param command Command to check against
	 * @return `true` if request is granted, `false` if denied.
	 */
	public static boolean canRunCommand(IMessage request, Command command)
	{
		IUser requestAuthor = request.getAuthor();
		IGuild originatedGuild;
		try
		{
			originatedGuild = request.getGuild();
		}
		catch(UnsupportedOperationException uoe) // DM
		{
			originatedGuild = null;
		}
		return canRunCommand(requestAuthor, originatedGuild, command);
	}
}
