package net.pingex.dcf.util;

import net.jodah.failsafe.*;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import sx.blah.discord.api.IDiscordClient;
import sx.blah.discord.handle.obj.IChannel;
import sx.blah.discord.handle.obj.IMessage;
import sx.blah.discord.util.DiscordException;
import sx.blah.discord.util.MissingPermissionsException;
import sx.blah.discord.util.RateLimitException;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * Contains some useful functions to avoid having to rewrite the same code over and over
 */
public class DiscordInteractionsUtil
{
	/**
	 * Logger
	 */
	private static final Logger LOGGER = LogManager.getLogger(DiscordInteractionsUtil.class);

	/**
	 * Maximum amount of retries for an interaction
	 */
	public static final int MAX_GW_RETRIES = 5;

	/**
	 * Delay between retries
	 */
	public static final int GW_RETRY_DELAY = 3;

	/**
	 * Retry policy for sending a message
	 */
	private static final RetryPolicy MESSAGE_RETRY_POLICY = new RetryPolicy()
			.retryOn(RateLimitException.class, DiscordException.class)
			.withDelay(GW_RETRY_DELAY, TimeUnit.SECONDS)
			.withMaxRetries(MAX_GW_RETRIES)
			.abortOn(MissingPermissionsException.class);

	/**
	 * Sends a message
	 * @param channel Channel to send the message
	 * @param message Message to send
	 * @return The sent message, or {@code Optional.empty()}
	 */
	public static Optional<IMessage> sendMessage(IChannel channel, String message)
	{
		LOGGER.debug("Attempting to send a message (error-tolerant operation).");

		SyncFailsafe fs = Failsafe.with(MESSAGE_RETRY_POLICY)
				.onFailedAttempt(failure -> LOGGER.warn("Failed to send message.", failure))
				.onRetry((c, f, stats) -> LOGGER.warn("Message sending failure #{}. Retrying.", stats.getExecutions()))
				.onSuccess((result, context) -> LOGGER.debug("Sent message after {} attempts.", context.getExecutions()))
				.onAbort(failure -> LOGGER.warn("Aborted sending message.", failure));

		try
		{
			return Optional.of((IMessage) fs.get(() -> channel.sendMessage(message)));
		}
		catch(FailsafeException e)
		{
			LOGGER.error("Failed to send message (exceeded amounts or trials or aborted)", e);
			return Optional.empty();
		}
	}

	/**
	 * Policy for a gateway disconnection
	 */
	private static final RetryPolicy GW_LOGOUT_POLICY = new RetryPolicy()
			.retryOn(RateLimitException.class, DiscordException.class)
			.withDelay(GW_RETRY_DELAY, TimeUnit.SECONDS)
			.withMaxRetries(MAX_GW_RETRIES);

	/**
	 * Disconnect from the gateway
	 * @param client Client interface
	 */
	public static void disconnect(IDiscordClient client)
	{
		LOGGER.debug("Attempting to disconnect user #{} from the gateway (error-tolerant operation).", client.getOurUser().getID());

		SyncFailsafe fs = Failsafe.with(GW_LOGOUT_POLICY)
				.onFailedAttempt(failure -> LOGGER.warn("Failed to disconnect.", failure))
				.onRetry((result, failure, context) -> LOGGER.warn("Disconnection failure #{}. Retrying.", context.getExecutions()))
				.onSuccess((result, context) -> LOGGER.info("Disconnected user {} from gateway after {} attempts.", client.getOurUser().getID(), context.getExecutions()));

		try
		{
			fs.run(client::logout);
		}
		catch(FailsafeException e)
		{
			LOGGER.error("Connection #{} failed to disconnect (amount of trials exceeded)", e);
		}
	}
}
